/*
 * This file is part of PySide: Python for Qt
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * Contact: PySide team <contact@pyside.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */


//workaround to access protected functions
#define protected public

// default includes
#include <shiboken.h>
#include <pysidesignal.h>
#include <pysideproperty.h>
#include <pyside.h>
#include <destroylistener.h>
#include <typeresolver.h>
#include <typeinfo>
#include "pyside_qtgui_python.h"

#include "qpixmapcache_key_wrapper.h"

// Extra includes
#include <qpixmapcache.h>



// Target ---------------------------------------------------------

extern "C" {
static int
Sbk_QPixmapCache_Key_Init(PyObject* self, PyObject* args, PyObject* kwds)
{
    if (Shiboken::Object::isUserType(self) && !Shiboken::ObjectType::canCallConstructor(self->ob_type, Shiboken::SbkType<QPixmapCache::Key >()))
        return -1;

    QPixmapCache::Key* cptr = 0;
    int overloadId = -1;
    SbkObject* sbkSelf = reinterpret_cast<SbkObject*>(self);


    int numArgs = PyTuple_GET_SIZE(args);
    PyObject* pyargs[] = {0};

    // invalid argument lengths


    if (!PyArg_UnpackTuple(args, "Key", 0, 1, &(pyargs[0])))
        return -1;

    // Overloaded function decisor
    // 0: Key()
    // 1: Key(QPixmapCache::Key)
    if (numArgs == 0) {
        overloadId = 0; // Key()
    } else if (numArgs == 1 && Shiboken::Converter< ::QPixmapCache::Key >::isConvertible(pyargs[0])) {
        overloadId = 1; // Key(QPixmapCache::Key)
    }

    // Function signature not found.
    if (overloadId == -1) goto Sbk_QPixmapCache_Key_Init_TypeError;

    // Call function/method
    {
        switch (overloadId) {
            case 0: // Key()
            {

                if(!PyErr_Occurred()) {
                    // Key()
                    PyThreadState* _save = PyEval_SaveThread(); // Py_BEGIN_ALLOW_THREADS
                    cptr = new QPixmapCache::Key();
                    PyEval_RestoreThread(_save); // Py_END_ALLOW_THREADS
                }
                break;
            }
            case 1: // Key(QPixmapCache::Key)
            {
                if (!Shiboken::Object::isValid(pyargs[0]))
                    return -1;
                QPixmapCache::Key cppArg0 = Shiboken::Converter<QPixmapCache::Key >::toCpp(pyargs[0]);

                if(!PyErr_Occurred()) {
                    // Key(QPixmapCache::Key)
                    PyThreadState* _save = PyEval_SaveThread(); // Py_BEGIN_ALLOW_THREADS
                    cptr = new QPixmapCache::Key(cppArg0);
                    PyEval_RestoreThread(_save); // Py_END_ALLOW_THREADS
                }
                break;
            }
        }
    }

    if (PyErr_Occurred() || !Shiboken::Object::setCppPointer(sbkSelf, Shiboken::SbkType<QPixmapCache::Key >(), cptr)) {
        delete cptr;
        return -1;
    }
    if (!cptr) goto Sbk_QPixmapCache_Key_Init_TypeError;

    Shiboken::Object::setValidCpp(sbkSelf, true);
    Shiboken::BindingManager::instance().registerWrapper(sbkSelf, cptr);


    return 1;

    Sbk_QPixmapCache_Key_Init_TypeError:
        const char* overloads[] = {"", "PySide.QtGui.QPixmapCache::Key", 0};
        Shiboken::setErrorAboutWrongArguments(args, "PySide.QtGui.Key", overloads);
        return -1;
}

static PyObject *Sbk_QPixmapCache_Key___copy__(PyObject *self)
{
        QPixmapCache::Key* cppSelf = 0;
        if (!Shiboken::Object::isValid(self))
            return 0;
        cppSelf = Shiboken::Converter<QPixmapCache::Key*>::toCpp(self);
        PyObject* pyResult = 0;
        QPixmapCache::Key* copy = new QPixmapCache::Key(*cppSelf);
        pyResult = Shiboken::Converter<QPixmapCache::Key*>::toPython(copy);
        Shiboken::Object::getOwnership(pyResult);

        if (PyErr_Occurred() || !pyResult) {
            Py_XDECREF(pyResult);
            return 0;
        }
        return pyResult;
}

static PyMethodDef Sbk_QPixmapCache_Key_methods[] = {

    {"__copy__", (PyCFunction)Sbk_QPixmapCache_Key___copy__, METH_NOARGS},
    {0} // Sentinel
};

// Rich comparison
static PyObject* Sbk_QPixmapCache_Key_richcompare(PyObject* self, PyObject* arg, int op)
{
    PyObject* pyResult = 0;
    QPixmapCache::Key& cppSelf = *Shiboken::Converter< QPixmapCache::Key* >::toCpp(self);

    switch (op) {
        case Py_NE:
            if (Shiboken::Converter< ::QPixmapCache::Key >::isConvertible(arg)) {
                // operator!=(const QPixmapCache::Key & key) const
                QPixmapCache::Key cppArg0 = Shiboken::Converter< ::QPixmapCache::Key >::toCpp(arg);
                pyResult = Shiboken::Converter< bool >::toPython(cppSelf != cppArg0);
            } else {
                pyResult = Py_True;
                Py_INCREF(pyResult);
            }

            break;
        case Py_EQ:
            if (Shiboken::Converter< ::QPixmapCache::Key >::isConvertible(arg)) {
                // operator==(const QPixmapCache::Key & key) const
                QPixmapCache::Key cppArg0 = Shiboken::Converter< ::QPixmapCache::Key >::toCpp(arg);
                pyResult = Shiboken::Converter< bool >::toPython(cppSelf == cppArg0);
            } else {
                pyResult = Py_False;
                Py_INCREF(pyResult);
            }

            break;
        default:
            goto Sbk_QPixmapCache_Key_RichComparison_TypeError;
    }

    if (pyResult && !PyErr_Occurred())
        return pyResult;
    Sbk_QPixmapCache_Key_RichComparison_TypeError:
    PyErr_SetString(PyExc_NotImplementedError, "operator not implemented.");
    return 0;

}

} // extern "C"

static int Sbk_QPixmapCache_Key_traverse(PyObject* self, visitproc visit, void* arg)
{
    return reinterpret_cast<PyTypeObject*>(&SbkObject_Type)->tp_traverse(self, visit, arg);
}
static int Sbk_QPixmapCache_Key_clear(PyObject* self)
{
    return reinterpret_cast<PyTypeObject*>(&SbkObject_Type)->tp_clear(self);
}
// Class Definition -----------------------------------------------
extern "C" {
static SbkObjectType Sbk_QPixmapCache_Key_Type = { { {
    PyObject_HEAD_INIT(&SbkObjectType_Type)
    /*ob_size*/             0,
    /*tp_name*/             "PySide.QtGui.QPixmapCache.Key",
    /*tp_basicsize*/        sizeof(SbkObject),
    /*tp_itemsize*/         0,
    /*tp_dealloc*/          &SbkDeallocWrapper,
    /*tp_print*/            0,
    /*tp_getattr*/          0,
    /*tp_setattr*/          0,
    /*tp_compare*/          0,
    /*tp_repr*/             0,
    /*tp_as_number*/        0,
    /*tp_as_sequence*/      0,
    /*tp_as_mapping*/       0,
    /*tp_hash*/             0,
    /*tp_call*/             0,
    /*tp_str*/              0,
    /*tp_getattro*/         0,
    /*tp_setattro*/         0,
    /*tp_as_buffer*/        0,
    /*tp_flags*/            Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_GC,
    /*tp_doc*/              0,
    /*tp_traverse*/         Sbk_QPixmapCache_Key_traverse,
    /*tp_clear*/            Sbk_QPixmapCache_Key_clear,
    /*tp_richcompare*/      Sbk_QPixmapCache_Key_richcompare,
    /*tp_weaklistoffset*/   0,
    /*tp_iter*/             0,
    /*tp_iternext*/         0,
    /*tp_methods*/          Sbk_QPixmapCache_Key_methods,
    /*tp_members*/          0,
    /*tp_getset*/           0,
    /*tp_base*/             reinterpret_cast<PyTypeObject*>(&SbkObject_Type),
    /*tp_dict*/             0,
    /*tp_descr_get*/        0,
    /*tp_descr_set*/        0,
    /*tp_dictoffset*/       0,
    /*tp_init*/             Sbk_QPixmapCache_Key_Init,
    /*tp_alloc*/            0,
    /*tp_new*/              SbkObjectTpNew,
    /*tp_free*/             0,
    /*tp_is_gc*/            0,
    /*tp_bases*/            0,
    /*tp_mro*/              0,
    /*tp_cache*/            0,
    /*tp_subclasses*/       0,
    /*tp_weaklist*/         0
}, },
    /*priv_data*/           0
};
} //extern


void init_QPixmapCache_Key(PyObject* module)
{
    SbkPySide_QtGuiTypes[SBK_QPIXMAPCACHE_KEY_IDX] = reinterpret_cast<PyTypeObject*>(&Sbk_QPixmapCache_Key_Type);

    Shiboken::ObjectType::initPrivateData(&Sbk_QPixmapCache_Key_Type);
    Shiboken::ObjectType::setDestructorFunction(&Sbk_QPixmapCache_Key_Type, &Shiboken::callCppDestructor<QPixmapCache::Key >);
    Py_INCREF((PyObject*)&Sbk_QPixmapCache_Key_Type); //Incref due the 'PyModule_AddObject' steals the reference.
    if (PyType_Ready((PyTypeObject*)&Sbk_QPixmapCache_Key_Type) < 0)
        return;

    Shiboken::ObjectType::setOriginalName(&Sbk_QPixmapCache_Key_Type, "QPixmapCache::Key");
    PyDict_SetItemString(module,"Key", (PyObject*)&Sbk_QPixmapCache_Key_Type);

    Shiboken::TypeResolver::createValueTypeResolver<QPixmapCache::Key >("QPixmapCache::Key");
    Shiboken::TypeResolver::createObjectTypeResolver<QPixmapCache::Key >("QPixmapCache::Key*");
    Shiboken::TypeResolver::createValueTypeResolver<QPixmapCache::Key >(typeid(QPixmapCache::Key).name());
    qRegisterMetaType< ::QPixmapCache::Key >("Key");
    qRegisterMetaType< ::QPixmapCache::Key >("QPixmapCache::Key");
}

